package com.ebaiyihui.family.doctor.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.ebaiyihui.family.doctor.common.dto.NotifyConsultDataDTO;
import com.ebaiyihui.family.doctor.common.dto.SendImMsgDTO;
import com.ebaiyihui.family.doctor.common.dto.SignedDoctorDTO;
import com.ebaiyihui.family.doctor.server.common.constants.CommonConstants;
import com.ebaiyihui.family.doctor.server.common.enums.ImSignStatus;
import com.ebaiyihui.family.doctor.server.common.enums.SignStatus;
import com.ebaiyihui.family.doctor.server.common.enums.StatusEnum;
import com.ebaiyihui.family.doctor.server.entity.MobileBenefitPackageEntity;
import com.ebaiyihui.family.doctor.server.entity.PatientSignEntity;
import com.ebaiyihui.family.doctor.server.exception.BusinessException;
import com.ebaiyihui.family.doctor.server.mapper.MobileBenefitPackageMapper;
import com.ebaiyihui.family.doctor.server.mapper.PatientSignMapper;
import com.ebaiyihui.family.doctor.server.rabbitmq.RabbitProduct;
import com.ebaiyihui.family.doctor.server.service.GoEasyPushService;
import com.ebaiyihui.family.doctor.server.service.ImChatTemplate;
import com.ebaiyihui.family.doctor.server.service.ImMsgTemplateService;
import com.ebaiyihui.family.doctor.server.service.PatientService;
import com.ebaiyihui.family.doctor.server.util.DateUtils;
import com.ebaiyihui.family.doctor.server.util.ThirdOrderPushUtil;
import com.ebaiyihui.family.doctor.server.util.UUIDUtil;
import com.ebaiyihui.family.doctor.server.vo.OrderTaskVo;
import com.ebaiyihui.framework.response.BaseResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.util.Date;
import java.util.Objects;

/**
 * @ClassName: PatientServiceImpl
 * @Author:yanliang
 * @Date:2024/3/13 09:51
 * @Description
 */
@Service
@Slf4j
public class PatientServiceImpl implements PatientService {

    @Autowired
    private PatientSignMapper patientSignMapper;

    @Autowired
    private MobileBenefitPackageMapper mobileBenefitPackageMapper;

    @Autowired
    private ImMsgTemplateService imMsgTemplateService;

    @Autowired
    private GoEasyPushService goEasyPushService;

    @Autowired
    private ImChatTemplate imChatTemplate;

    @Autowired
    private ThirdOrderPushUtil thirdOrderPushUtil;

    @Autowired
    private RabbitProduct rabbitProduct;


    @Override
    public BaseResponse<String> signedDoctor(SignedDoctorDTO reqVo) {

        MobileBenefitPackageEntity entity = mobileBenefitPackageMapper.queryByPhone(reqVo.getPhone(), reqVo.getPackageOrderId());
        if (ObjectUtils.isEmpty(entity)) {
            return BaseResponse.error("没有查询到有关权益");
        }

        String admId = UUIDUtil.getUUID();
        Date curDate = new Date();
        Date signEndTime = DateUtils.getAfterDay(curDate, CommonConstants.DAY);
        // 处理换绑医生
        if (Objects.nonNull(reqVo.getAdmId())) {
            QueryWrapper<PatientSignEntity> psWrapper = new QueryWrapper<>();
            PatientSignEntity patientSign = new PatientSignEntity();
            patientSign.setAdmId(admId);
            psWrapper.setEntity(patientSign);
            PatientSignEntity patientSignEntity = patientSignMapper.selectOne(psWrapper);
            if (Objects.nonNull(patientSignEntity)) {
                signEndTime = patientSignEntity.getSignEndTime();
                patientSignEntity.setStatus(StatusEnum.FINISH_APPLY.getValue());
                patientSignEntity.setSignStatus(SignStatus.CANCEL.getValue());
                patientSignEntity.setSubStatus(2);
                patientSignEntity.setSignEndTime(curDate);
                patientSignMapper.updateById(patientSignEntity);
            }
        }

        PatientSignEntity patientSignEntity = new PatientSignEntity();
        patientSignEntity.setAdmId(admId);
        patientSignEntity.setAppCode(CommonConstants.APP_CODE);
        patientSignEntity.setOrganId(CommonConstants.ORGAN_CODE);
        patientSignEntity.setOrganName(CommonConstants.ORGAN_NAME);
        patientSignEntity.setDoctorId(reqVo.getDoctorId());
        patientSignEntity.setDoctorName(reqVo.getDoctorName());
        patientSignEntity.setDeptId(reqVo.getDeptId());
        patientSignEntity.setDeptName(reqVo.getDeptName());
        patientSignEntity.setSignStatus(SignStatus.CANCEL.getValue().equals(reqVo.getSignStatus()) ? SignStatus.SIGNED.getValue() : reqVo.getSignStatus());
        patientSignEntity.setPatientPhone(reqVo.getPhone());
        patientSignEntity.setCredNo(reqVo.getCredNo());
        patientSignEntity.setPatientId(reqVo.getPatientId());
        patientSignEntity.setPatientName(reqVo.getName());
        patientSignEntity.setPatientUserId(entity.getUserId());
        patientSignEntity.setStatus(StatusEnum.IN_CONSULTATION.getValue());
        patientSignEntity.setSignEndTime(signEndTime);

        patientSignMapper.insert(patientSignEntity);

        //创建IM
        BaseResponse createImResponse = imChatTemplate.createImSession(patientSignEntity);
        if (!createImResponse.isSuccess()) {
            log.info("创建IM失败:{}", createImResponse.getMsg());
            throw new BusinessException("创建IM失败");
        }

        SendImMsgDTO sendImMsgDTO = new SendImMsgDTO();

        // 推送im主动消息
        sendImMsgDTO.setAdmId(admId);
        sendImMsgDTO.setAppCode(CommonConstants.APP_CODE);
        sendImMsgDTO.setOrganId(CommonConstants.ORGAN_CODE);
        sendImMsgDTO.setType(SignStatus.SIGNED.getValue().equals(reqVo.getSignStatus()) ? ImSignStatus.SIGN_SUC.getValue() :
                SignStatus.CANCEL.getValue().equals(reqVo.getSignStatus()) ? ImSignStatus.REISSUE_SUC.getValue() :
                        ImSignStatus.INQUIRY_SCH_DOC.getValue());

        if (!SignStatus.NORMAL.getValue().equals(reqVo.getSignStatus())) {
            // 推送第三方权益使用
            NotifyConsultDataDTO notifyConsultDataDTO = new NotifyConsultDataDTO();
            notifyConsultDataDTO.setActivateOrderId(reqVo.getPackageOrderId());
            notifyConsultDataDTO.setConsultOrderNo(admId);
            notifyConsultDataDTO.setStatus(String.valueOf(reqVo.getSignStatus()));
            notifyConsultDataDTO.setProductId(entity.getProductId());
            notifyConsultDataDTO.setPatientId(entity.getUserId());
            notifyConsultDataDTO.setExpertName(reqVo.getDoctorName());
            notifyConsultDataDTO.setOrderDate(DateUtils.dateToFullString(patientSignEntity.getCreateTime()));
            thirdOrderPushUtil.syncBenefitUsedOrder(notifyConsultDataDTO);
        } else {
            QueryWrapper<PatientSignEntity> psWrapper = new QueryWrapper<>();
            PatientSignEntity patientSign = new PatientSignEntity();
            if (Objects.nonNull(reqVo.getPhone())) {
                patientSign.setPatientPhone(reqVo.getPhone());
            }
            psWrapper.setEntity(patientSign);
            if (Objects.nonNull(reqVo.getPhone())) {
                psWrapper.orderByDesc("createTime");
                psWrapper.last("limit 1");
            }
            PatientSignEntity pse = patientSignMapper.selectOne(psWrapper);
            if (null == pse) {
                throw new BusinessException("未查询到签约信息,不能获取排班医生！");
            }
            sendImMsgDTO.setOldAdmId(pse.getAdmId());

            //放入MQ任务,处理订单到时结束
            OrderTaskVo orderTaskVo = new OrderTaskVo();
            orderTaskVo.setType(1);
            orderTaskVo.setId(String.valueOf(patientSignEntity.getId()));
            rabbitProduct.sendDelay(orderTaskVo);
        }

        // 推送第三方签名相关订单信息
        thirdOrderPushUtil.pushSignedOrder(String.valueOf(reqVo.getDoctorId()), reqVo.getPhone(), String.valueOf(patientSignEntity.getId()), reqVo.getPackageOrderId(), reqVo.getSignStatus());

        imMsgTemplateService.sendImMsg(sendImMsgDTO);

        // 推送app提示语
        goEasyPushService.newOrderToReceive(admId);

        return BaseResponse.success(admId);
    }

    @Override
    public BaseResponse<PatientSignEntity> getSignedDoctor(SignedDoctorDTO reqVo) {
        QueryWrapper<PatientSignEntity> psWrapper = new QueryWrapper<>();
        PatientSignEntity patientSign = new PatientSignEntity();
        if (Objects.nonNull(reqVo.getAdmId())) {
            patientSign.setAdmId(reqVo.getAdmId());
        }
        if (Objects.nonNull(reqVo.getPhone())) {
            patientSign.setPatientPhone(reqVo.getPhone());
        }
        psWrapper.setEntity(patientSign);
        if (Objects.nonNull(reqVo.getPhone())) {
            psWrapper.orderByDesc("createTime");
            psWrapper.last("limit 1");
        }

        PatientSignEntity patientSignEntity = patientSignMapper.selectOne(psWrapper);
        if (null == patientSignEntity) {
            throw new BusinessException("未查询到签约医生信息！");
        }
        return BaseResponse.success(patientSignEntity);
    }
}
